#include <vector>
#include <algorithm>
#include <set>

#include<vcg/simplex/vertex/base.h>
#include<vcg/simplex/face/base.h>
#include<vcg/simplex/face/component_ocf.h>
#include<vcg/complex/trimesh/base.h>

#include<vcg/complex/trimesh/update/flag.h>
#include<vcg/simplex/face/pos.h>
#include<vcg/complex/trimesh/update/topology.h>

#include<wrap/io_trimesh/import_ply.h>
#include<wrap/io_trimesh/export_ply.h>

using namespace vcg;

class MyEdge;    // dummy prototype never used
class MyFace;
class MyFaceO;
class MyVertex;

class MyVertexO : public VertexSimp2< MyVertexO, MyEdge, MyFaceO, vertex::Coord3f, vertex::BitFlags, vertex::Color4b, vertex::VFAdj >{};
class MyFaceO    : public FaceSimp2  < MyVertexO, MyEdge, MyFaceO, face::VertexRef, face::InfoOcf , face::BitFlags, face::FFAdjOcf> {};
class MyMeshO : public tri::TriMesh< std::vector<MyVertexO>, face::vector_ocf<MyFaceO> >{};

class MyVertex  : public VertexSimp2< MyVertex, MyEdge, MyFace, vertex::Coord3f, vertex::BitFlags, vertex::Color4b, vertex::VFAdj >{};
class MyFace    : public FaceSimp2  < MyVertex, MyEdge, MyFace, face::VertexRef, face::BitFlags, face::FFAdj> {};

class MyMesh : public tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace > >{};


int main(int argc,char **argv ){

    MyMesh m;
    MyMeshO mo;
    if(argc==1) return 0;

	if(tri::io::ImporterPLY<MyMesh>::Open(m,argv[1])!=0)
		{
      printf("Error reading file  %s\n",argv[1]);
			exit(0);
		}
		
    if(tri::io::ImporterPLY<MyMeshO>::Open(mo,argv[1])!=0)
        {
      printf("Error reading file  %s\n",argv[1]);
            exit(0);
        }

    printf("vn %i fn %i\n",m.vn, m.fn);

    MyMesh::VertexIterator vi;
    MyMesh::FaceIterator fi;
    int t0=clock();
    //mo.face.EnableFFAdjacency();
    vcg::tri::UpdateTopology<MyMeshO>::FaceFace(mo);
    int t1=clock();

    int t2=clock();
    vcg::tri::UpdateTopology<MyMesh>::FaceFace(m);
    int t3=clock();

    printf("%i %i\n",t1-t0, t3-t2 );
  return 0;
}

